/* ELVIS code */

/* general naming conventions:
un: unobservables
ob: observables
theta: parameter vector
nuis: nuissance parameters (profiled over)
gam: tilting parameter gamma (see paper for details)
*/

/* if you want to use EL comment out the next line */
#define DOGMM

/* User should edit this file to implement their model 
   Default values below are for 
   Example 1 (continued) in the paper, Equation (18)
   (Interval valued data model with additional moment conditions).
   Note that you may get slightly different plots than in the paper due to
   differences in random number generators across version of gauss and/or ox
   (this is a small sample).
*/

/* define the moment function */
/* un: unobservables, ob: observables, theta: parameter vector, including nuissance param */
/* un and ob are for one individual, not the whole sample */
proc(1)=mom(un,ob,theta);
  local g;
  local yt,dy,x,y;
  x=ob[1];
  y=ob[2];
  yt=y+un*1.0;
  dy=yt-theta[1]*x;
  g= dy | x*dy | (dy^2-theta[2]) | x^2*(dy^2-theta[2]) ;
  retp(g);
endp;

/* indicate the number of nuisance parameters (to be profiled out) */
/* the last getnbnuis() entries of theta will be profiled out */
proc(1)=getnbnuis();
 retp(1);
endp;

/* user-specified measure rho */
proc(1)=rho(un,ob,theta);
  retp(1);
endp;

/* initial guess for the unobservable */
/* may be a function of obervables, theta and gam but does not need to be */
proc(1)=guess_un(ob,theta,gam);
  retp(1.0*rndu(1,1));
endp;

/* rule for trial jumps in Metropolis Monte Carlo */
proc(1)=jump_un(un,ob,theta);
  retp(1.0*rndu(1,1));
endp;

/* load the library  - no need to edit elvisutil.g */
#include "elvisutil.g"

proc(1)=mymain();
  local n,theta,x,y,l,thetao,dy,yt,ymax,ymin,res;
  local ptol,ftol,rep,tol,myseed;
  local nuisgam;

/* numerical parameters */
  ptol=2.0;
  ftol=1e-3;
  rep=50|500;
  tol=1e-4;
  myseed=105;

/* data generation */
  res=1.0;
  n=250;
  thetao={1};
  x=1.0*rndn(n,1);
  dy=0.5*rndn(n,1);
  yt=thetao[1]*x+dy;
  y=res*int(yt/res);

/* simple Manski-type bounds */
  print "Manski-type Bounds";
  ymax=(x.>0).*(y+res)+(x.<0).*(y);
  ymin=(x.>0).*(y)+(x.<0).*(y+res);
  print "max";
  print ymax/x;
  print "min";
  print ymin/x;

/* calculate ELVIS objective function over a set of grid points */
  print "ELVIS";
  print "theta   L(theta)";
  theta=0.0;
  do while (theta <= 2.01);
    nuisgam=0.2; /* initial guess for theta_2 */
    {l,nuisgam}=calc_lnL_elvis(x~y,theta,nuisgam,ptol,ftol,rep,tol,myseed);
    if (oxmpi_ismaster()); print theta~l; endif;
    theta=theta+0.1;
  endo;
endp;

mymain();
